@@ -4,12 +4,12 @@ where
 -- version number is major.minor.patchlvl; don't change the format of the
 -- `versnum' line as it is `grep'ed for by a Makefile
 --
-idstr      = "$Id: BaseVersion.hs,v 1.43 2003/10/19 10:45:27 chak Exp $"
+idstr      = "$Id: BaseVersion.hs,v 1.44 2005/05/18 03:04:03 chak Exp $"
 name       = "Compiler Toolkit"
-versnum    = "0.26.0"
-date	   = "19 Oct 2003"
+versnum    = "0.27.0"
+date	   = "18 May 2005"
 version    = name ++ ", version " ++ versnum ++ ", " ++ date
-copyright  = "Copyright (c) [1995..2003] Manuel M T Chakravarty"
+copyright  = "Copyright (c) [1995..2005] Manuel M T Chakravarty"
 disclaimer = "This software is distributed under the \
 	     \terms of the GNU Public Licence.\n\
 	     \NO WARRANTY WHATSOEVER IS PROVIDED. \
