@@ -75,7 +75,7 @@ module CTrav (CT, readCT, transCT, getCHeaderCT, runCT, throwCTExc, ifCTExc,
 	      declaredDeclr, declaredName, structMembers, expandDecl,
 	      structName, enumName, tagName, isPtrDeclr, dropPtrDeclr,
 	      isPtrDecl, isFunDeclr, structFromDecl, funResultAndArgs,
-	      chaseDecl, findAndChaseDecl, checkForAlias,
+	      chaseDecl, findAndChaseDecl, checkForAlias, checkForOneCUName,
 	      checkForOneAliasName, lookupEnum, lookupStructUnion,
 	      lookupDeclOrTag)
 where
@@ -687,6 +687,16 @@ checkForAlias decl  =
 checkForOneAliasName      :: CDecl -> Maybe Ident
 checkForOneAliasName decl  = fmap fst $ extractAlias decl False
 
+-- given a declaration, find the name of the struct/union type
+checkForOneCUName        :: CDecl -> Maybe Ident
+checkForOneCUName decl@(CDecl specs _ _)  = 
+  case [ts | CTypeSpec ts <- specs] of
+    [CSUType (CStruct _ n _ _) _] -> 
+	case declaredDeclr decl of
+	  Nothing			-> n
+          Just (CVarDeclr _ _         ) -> n
+	  Just (CPtrDeclr [_] (CVarDeclr _ _) _) -> Nothing
+    _                                  -> Nothing
 
 -- smart lookup
 --
