@@ -5,7 +5,7 @@
 --
 --  Version $Revision: 1.27 $ from $Date: 2005/03/14 00:26:58 $
 --
---  Copyright (c) [1999..2004] Manuel M T Chakravarty
+--  Copyright (c) [1999..2005] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
 --  it under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@
 --	      | `fun' [`pure'] [`unsafe'] idalias parms
 --	      | `get' apath
 --	      | `set' apath
---	      | `pointer' ['*'] idalias ptrkind
+--	      | `pointer' ['*'] idalias ptrkind ['nocode']
 --	      | `class' [ident `=>'] ident ident
 --  ctxt     -> [`lib' `=' string] [prefix]
 --  idalias  -> ident [`as' (ident | `^')]
@@ -71,12 +71,14 @@
 --	      | apath `.' ident
 --	      | apath `->' ident
 --  trans    -> `{' alias_1 `,' ... `,' alias_n `}'
---  alias    -> `underscoreToCase'
+--  alias    -> `underscoreToCase' | `upcaseFirstLetter' 
+--	      | `downcaseFirstLetter'
 --	      | ident `as' ident
 --  ptrkind  -> [`foreign' | `stable'] ['newtype' | '->' ident]
 --  
---  If `underscoreToCase' occurs in a translation table, it must be the first
---  entry.
+--  If `underscoreToCase', `upcaseFirstLetter', or `downcaseFirstLetter'
+--  occurs in a translation table, it must be the first entry, or if two of
+--  them occur the first two entries.
 --
 --  Remark: Optional Haskell names are normalised during structure tree
 --	    construction, ie, associations that associated a name with itself
@@ -86,10 +88,11 @@
 --- TODO ----------------------------------------------------------------------
 --
 
-module CHS (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..), CHSParm(..),
-	    CHSArg(..), CHSAccess(..), CHSAPath(..), CHSPtrType(..),
-	    loadCHS, dumpCHS, hssuffix, chssuffix, loadCHI, dumpCHI,
-	    chisuffix, showCHSParm)
+module CHS (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
+	    CHSChangeCase(..), CHSParm(..), CHSArg(..), CHSAccess(..),
+	    CHSAPath(..), CHSPtrType(..), 
+	    loadCHS, dumpCHS, hssuffix, chssuffix, loadCHI, dumpCHI, chisuffix,
+	    showCHSParm)
 where 
 
 -- standard libraries
@@ -194,6 +197,7 @@ data CHSHook = CHSImport  Bool			-- qualified?
 			  CHSPtrType		-- Ptr, ForeignPtr or StablePtr
 			  Bool			-- create new type?
 			  (Maybe Ident)		-- Haskell type pointed to
+			  Bool			-- emit type decl?
 			  Position
              | CHSClass   (Maybe Ident)		-- superclass
 			  Ident			-- class name
@@ -209,7 +213,7 @@ instance Pos CHSHook where
   posOf (CHSCall    _ _ _ _       pos) = pos
   posOf (CHSFun     _ _ _ _ _ _ _ pos) = pos
   posOf (CHSField   _ _           pos) = pos
-  posOf (CHSPointer _ _ _ _ _ _   pos) = pos
+  posOf (CHSPointer _ _ _ _ _ _ _ pos) = pos
   posOf (CHSClass   _ _ _	  pos) = pos
 
 -- two hooks are equal if they have the same Haskell name and reference the
@@ -233,8 +237,8 @@ instance Eq CHSHook where
     oalias1 == oalias2 && ide1 == ide2
   (CHSField acc1 path1         _) == (CHSField acc2 path2         _) =    
     acc1 == acc2 && path1 == path2
-  (CHSPointer _ ide1 oalias1 _ _ _ _) 
-			          == (CHSPointer _ ide2 oalias2 _ _ _ _) =
+  (CHSPointer _ ide1 oalias1 _ _ _ _ _) 
+			          == (CHSPointer _ ide2 oalias2 _ _ _ _ _) =
     ide1 == ide2 && oalias1 == oalias2
   (CHSClass _ ide1 _           _) == (CHSClass _ ide2 _           _) =
     ide1 == ide2
@@ -243,8 +247,14 @@ instance Eq CHSHook where
 -- translation table (EXPORTED)
 --
 data CHSTrans = CHSTrans Bool			-- underscore to case?
+			 CHSChangeCase		-- upcase or downcase?
 			 [(Ident, Ident)]	-- alias list
 
+data CHSChangeCase = CHSSameCase 
+		   | CHSUpCase 
+		   | CHSDownCase
+		   deriving Eq
+
 -- marshalling descriptor for function hooks (EXPORTED)
 --
 -- * a marshaller consists of a function name and flag indicating whether it
@@ -481,7 +491,7 @@ showCHSHook (CHSField acc path _) =
        CHSGet -> showString "get "
        CHSSet -> showString "set ")
   . showCHSAPath path
-showCHSHook (CHSPointer star ide oalias ptrType isNewtype oRefType _) =
+showCHSHook (CHSPointer star ide oalias ptrType isNewtype oRefType emit _) =
     showString "pointer "
   . (if star then showString "*" else showString "")
   . showIdAlias ide oalias
@@ -493,6 +503,9 @@ showCHSHook (CHSPointer star ide oalias ptrType isNewtype oRefType _) =
        (True , _       ) -> showString " newtype" 
        (False, Just ide) -> showString " -> " . showCHSIdent ide
        (False, Nothing ) -> showString "")
+  . (case emit of
+       True  -> showString "" 
+       False -> showString " nocode")
 showCHSHook (CHSClass oclassIde classIde typeIde _) =   
     showString "class "
   . (case oclassIde of
@@ -536,10 +549,11 @@ showCHSParm (CHSParm oimMarsh hsTyStr twoCVals oomMarsh _)  =
     --
     showHsVerb str = showChar '`' . showString str . showChar '\''
 
-showCHSTrans                          :: CHSTrans -> ShowS
-showCHSTrans (CHSTrans _2Case assocs)  =   
+showCHSTrans :: CHSTrans -> ShowS
+showCHSTrans (CHSTrans _2Case chgCase assocs)  =   
     showString "{"
   . (if _2Case then showString ("underscoreToCase" ++ maybeComma) else id)
+  . showCHSChangeCase chgCase
   . foldr (.) id (intersperse (showString ", ") (map showAssoc assocs))
   . showString "}"
   where
@@ -550,6 +564,11 @@ showCHSTrans (CHSTrans _2Case assocs)  =
       . showString " as "
       . showCHSIdent ide2
 
+showCHSChangeCase :: CHSChangeCase -> ShowS
+showCHSChangeCase CHSSameCase = id
+showCHSChangeCase CHSUpCase   = showString "upcaseFirstLetter"
+showCHSChangeCase CHSDownCase = showString "downcaseFirstLetter"
+
 showCHSAPath :: CHSAPath -> ShowS
 showCHSAPath (CHSRoot ide) =
   showCHSIdent ide
@@ -755,14 +774,32 @@ parseImport :: Position -> [CHSToken] -> CST s [CHSFrag]
 parseImport pos toks = do
   (qual, modid, toks') <- 
     case toks of
-      CHSTokIdent _ ide                :toks -> return (False, ide, toks)
-      CHSTokQualif _: CHSTokIdent _ ide:toks -> return (True , ide, toks)
+      CHSTokIdent _ ide                :toks ->
+        let (ide', toks') = rebuildModuleId ide toks
+         in return (False, ide', toks')
+      CHSTokQualif _: CHSTokIdent _ ide:toks ->
+        let (ide', toks') = rebuildModuleId ide toks
+         in return (True , ide', toks')
       _					     -> syntaxError toks
-  chi <- loadCHI . identToLexeme $ modid
+  chi <- loadCHI . moduleNameToFileName . identToLexeme $ modid
   toks'' <- parseEndHook toks'
   frags <- parseFrags toks''
   return $ CHSHook (CHSImport qual modid chi pos) : frags
 
+-- Qualified module names do not get lexed as a single token so we need to
+-- reconstruct it from a sequence of identifer and dot tokens.
+--
+rebuildModuleId ide (CHSTokDot _ : CHSTokIdent _ ide' : toks) = 
+  let catIdent ide ide' = onlyPosIdent (posOf ide)  --FIXME: unpleasent hack
+                            (identToLexeme ide ++ '.' : identToLexeme ide')
+   in rebuildModuleId (catIdent ide ide') toks
+rebuildModuleId ide                                     toks  = (ide, toks)
+
+moduleNameToFileName :: String -> FilePath
+moduleNameToFileName = map dotToSlash
+  where dotToSlash '.' = '/'
+        dotToSlash c   = c
+
 parseContext          :: Position -> [CHSToken] -> CST s [CHSFrag]
 parseContext pos toks  = do
 		           (olib    , toks'' ) <- parseOptLib          toks
@@ -916,11 +953,17 @@ parsePointer pos toks =
 	CHSTokNewtype _                  :toks' -> (True , Nothing , toks' )
 	CHSTokArrow   _:CHSTokIdent _ ide:toks' -> (False, Just ide, toks' )
 	_				        -> (False, Nothing , toks'3)
-    toks'5	                  <- parseEndHook toks'4
-    frags                         <- parseFrags   toks'5
+    let 
+     (emit, toks'5) =
+      case toks'4 of
+	CHSTokNocode _                  :toks' -> (False, toks' )
+	_                                      -> (True , toks'4 )
+    toks'6	                  <- parseEndHook toks'5
+    frags                         <- parseFrags   toks'6
     return $ 
       CHSHook 
-       (CHSPointer isStar ide (norm ide oalias) ptrType isNewtype oRefType pos)
+       (CHSPointer 
+         isStar ide (norm ide oalias) ptrType isNewtype oRefType emit pos)
        : frags
   where
     parsePtrType :: [CHSToken] -> CST s (CHSPtrType, [CHSToken])
@@ -1041,21 +1084,35 @@ parsePath' toks =
 parseTrans :: [CHSToken] -> CST s (CHSTrans, [CHSToken])
 parseTrans (CHSTokLBrace _:toks) =
   do
-    (_2Case, toks' ) <- parse_2Case toks
+    (_2Case, chgCase, toks' ) <- parse_2CaseAndChange toks
     case toks' of
-      (CHSTokRBrace _:toks'') -> return (CHSTrans _2Case [], toks'')
+      (CHSTokRBrace _:toks'') -> return (CHSTrans _2Case chgCase [], toks'')
       _			      ->
         do
 	  -- if there was no `underscoreToCase', we add a comma token to meet
 	  -- the invariant of `parseTranss'
 	  --
-	  (transs, toks'') <- if _2Case 
+	  (transs, toks'') <- if (_2Case || chgCase /= CHSSameCase)
 			      then parseTranss toks'
 			      else parseTranss (CHSTokComma nopos:toks')
-          return (CHSTrans _2Case transs, toks'')
+          return (CHSTrans _2Case chgCase transs, toks'')
   where
-    parse_2Case (CHSTok_2Case _:toks) = return (True, toks)
-    parse_2Case toks		      = return (False, toks)
+    parse_2CaseAndChange (CHSTok_2Case _:CHSTokComma _:CHSTokUpper _:toks) = 
+      return (True, CHSUpCase, toks)
+    parse_2CaseAndChange (CHSTok_2Case _:CHSTokComma _:CHSTokDown _ :toks) = 
+      return (True, CHSDownCase, toks)
+    parse_2CaseAndChange (CHSTok_2Case _                            :toks) = 
+      return (True, CHSSameCase, toks)
+    parse_2CaseAndChange (CHSTokUpper _:CHSTokComma _:CHSTok_2Case _:toks) = 
+      return (True, CHSUpCase, toks)
+    parse_2CaseAndChange (CHSTokUpper _                             :toks) = 
+      return (False, CHSUpCase, toks)
+    parse_2CaseAndChange (CHSTokDown  _:CHSTokComma _:CHSTok_2Case _:toks) = 
+      return (True, CHSDownCase, toks)
+    parse_2CaseAndChange (CHSTokDown  _                             :toks) = 
+      return (False, CHSDownCase, toks)
+    parse_2CaseAndChange toks		                                   = 
+      return (False, CHSSameCase, toks)
     --
     parseTranss (CHSTokRBrace _:toks) = return ([], toks)
     parseTranss (CHSTokComma  _:toks) = do
