@@ -1,10 +1,5 @@
 --  C->Haskell Compiler: binding generator
 --
---  Author : Manuel M T Chakravarty
---  Created: 17 August 99
---
---  Version $Revision: 1.54 $ from $Date: 2005/03/14 00:26:58 $
---
 --  Copyright (c) [1999..2003] Manuel M T Chakravarty
 --
 --  This file is free software; you can redistribute it and/or modify
@@ -17,14 +12,12 @@
 --  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 --  GNU General Public License for more details.
 --
---- DESCRIPTION ---------------------------------------------------------------
+--- Description ---------------------------------------------------------------
+--
+--  Language: Haskell 98
 --
 --  Module implementing the expansion of the binding hooks.
 --
---- DOCU ----------------------------------------------------------------------
---
---  language: Haskell 98
---
 --  * If there is an error in one binding hook, it is skipped and the next one 
 --    is processed (to collect as many errors as possible).  However, if at
 --    least one error occured, the expansion of binding hooks ends in a fatal
@@ -91,7 +84,7 @@
 --  another identifier casts a shadow that matches.  If so, that identifier is
 --  taken instead of the original one.
 --
---- TODO ----------------------------------------------------------------------
+--- ToDo ----------------------------------------------------------------------
 --
 --  * A function prototype that uses a defined type on its left hand side may
 --    declare a function, while that is not obvious from the declaration
@@ -125,7 +118,7 @@ import Idents     (Ident, identToLexeme, onlyPosIdent)
 import Attributes (newAttrsOnlyPos)
 
 -- C->Haskell
-import C2HSConfig (dlsuffix)
+import C2HSConfig (PlatformSpec(..))
 import C2HSState  (CST, nop, errorsPresent, showErrors, fatal,
 		   SwitchBoard(..), Traces(..), putTraceStr, getSwitch,
 		   printCIO)
@@ -141,8 +134,8 @@ import C	  (AttrC, CObj(..), CTag(..), lookupDefObjC, lookupDefTagC,
 		   simplifyDecl, declrFromDecl, declrNamed, structMembers,
 		   structName, tagName, declaredName , structFromDecl,
 		   funResultAndArgs, chaseDecl, findAndChaseDecl,
-		   checkForAlias, checkForOneAliasName, lookupEnum,
-		   lookupStructUnion, lookupDeclOrTag, isPtrDeclr,
+		   checkForAlias, checkForOneAliasName, checkForOneCUName, 
+		   lookupEnum, lookupStructUnion, lookupDeclOrTag, isPtrDeclr,
 		   dropPtrDeclr, isPtrDecl, getDeclOf, isFunDeclr,
 		   refersToNewDef, CDef(..))
 
@@ -150,8 +143,7 @@ import C	  (AttrC, CObj(..), CTag(..), lookupDefObjC, lookupDefTagC,
 import CHS	  (CHSModule(..), CHSFrag(..), CHSHook(..), CHSTrans(..),
 		   CHSParm(..), CHSArg(..), CHSAccess(..), CHSAPath(..),
 		   CHSPtrType(..), showCHSParm) 
-import CInfo      (CPrimType(..), size, alignment, bitfieldIntSigned,
-		   bitfieldAlignment)
+import CInfo      (CPrimType(..), size, alignment, getPlatform)
 import GBMonad    (TransFun, transTabToTransFun, HsObject(..), GB,
 		   initialGBState, setContext, getPrefix, 
 		   delayCode, getDelayedCode, ptrMapsTo, queryPtr, objIs,
@@ -474,7 +466,8 @@ expandHook (CHSField access path pos) =
 					++ show (length offsets) ++ "\n"
     traceValueType et  = traceGenBind $ 
       "Type of accessed value: " ++ showExtType et ++ "\n"
-expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType pos) =
+expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType emit 
+	      pos) =
   do
     traceInfoPointer
     let hsIde  = fromMaybe cName oalias
@@ -500,6 +493,7 @@ expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType pos) =
 	    Nothing     -> do
 			     cDecl <- chaseDecl cNameFull (not isStar)
 			     et    <- extractPtrType cDecl
+			     traceInfoPtrType et
 			     let et' = adjustPtr isStar et
 			     return (showExtType et', isFunExtType et')
 	    Just hsType -> return (identToLexeme hsType, False)
@@ -508,7 +502,7 @@ expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType pos) =
 	    --   allow `... -> fun HSTYPE' to explicitly mark function
 	    --   types if this ever becomes important
 	traceInfoHsType hsName hsType
-	pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun
+	pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun emit
       Right tag -> do			        -- found a tag definition
         let cNameFull = tagName tag
 	traceInfoCName "tag definition" cNameFull
@@ -518,20 +512,25 @@ expandHook (CHSPointer isStar cName oalias ptrKind isNewtype oRefType pos) =
 		       Nothing     -> "()"
 		       Just hsType -> identToLexeme hsType
 	traceInfoHsType hsName hsType
-	pointerDef isStar cNameFull hsName ptrKind isNewtype hsType False
+	pointerDef isStar cNameFull hsName ptrKind isNewtype hsType False emit
   where
     -- remove a pointer level if the first argument is `False'
     --
-    adjustPtr True  et         = et
-    adjustPtr False (PtrET et) = et
-    adjustPtr _	    _	       = interr "GenBind.adjustPtr: Where is the Ptr?"
+    adjustPtr True  et                 = et
+    adjustPtr False (PtrET et)         = et
+    adjustPtr False et@(DefinedET _ _) = 
+      interr "GenBind.adjustPtr: Can't adjust defined type"
+    adjustPtr _	    _	               = 
+      interr "GenBind.adjustPtr: Where is the Ptr?"
     --
     traceInfoPointer        = traceGenBind "** Pointer hook:\n"
-    traceInfoCName kind ide = traceGenBind $ 
-      "found C " ++ kind ++ " for `" ++ identToLexeme ide ++ "'\n"
+    traceInfoPtrType et     = traceGenBind $ 
+      "extracted ptr type is `" ++ showExtType et ++ "'\n"
     traceInfoHsType name ty = traceGenBind $ 
       "associated with Haskell entity `" ++ name ++ "'\nhaving type " ++ ty 
       ++ "\n"
+    traceInfoCName kind ide = traceGenBind $ 
+      "found C " ++ kind ++ " for `" ++ identToLexeme ide ++ "'\n"
 expandHook (CHSClass oclassIde classIde typeIde pos) =
   do
     traceInfoClass
@@ -1072,13 +1071,11 @@ pointerDef :: Bool		-- explicit `*' in pointer hook
 	   -> Bool		-- explicit newtype tag
 	   -> String		-- Haskell type expression of pointer argument
 	   -> Bool		-- do we have a pointer to a function?
+	   -> Bool		-- shall we emit code?
 	   -> GB String
-pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun =
+pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun emit =
   do
-    keepOld <- getSwitch oldFFI
-    let ptrArg  = if keepOld 
-		  then "()"		-- legacy FFI interface
-		  else if isNewtype 
+    let ptrArg  = if isNewtype 
 		  then hsName		-- abstract type
 		  else hsType		-- concrete type
         ptrCon  = case ptrKind of
@@ -1091,10 +1088,13 @@ pointerDef isStar cNameFull hsName ptrKind isNewtype hsType isFun =
 					   "Ptr (" ++ ptrArg ++ ")")
       _		    -> thePtr `ptrMapsTo` (hsName, hsName)
     return $
-      if isNewtype 
-      then "newtype " ++ hsName ++ " = " ++ hsName ++ " (" ++ ptrType ++ ")" ++
+      case (emit, isNewtype) of
+        (False, _)     -> ""	-- suppress code generation
+	(True , True)  -> 
+	  "newtype " ++ hsName ++ " = " ++ hsName ++ " (" ++ ptrType ++ ")" ++
 	   withForeignFun
-      else "type "    ++ hsName ++ " = "                   ++ ptrType
+	(True , False) -> 
+	  "type "    ++ hsName ++ " = "                   ++ ptrType
     where
       -- if we have a foreign pointer wrapped into a newtype, provide a
       -- safe unwrapping function automatically
@@ -1305,7 +1305,7 @@ extractSimpleType                    :: Bool -> Position -> CDecl -> GB ExtType
 extractSimpleType isResult pos cdecl  =
   do
     traceEnter
-    ct <- extractCompType isResult cdecl
+    ct <- extractCompType isResult True cdecl
     case ct of
       ExtType et -> return et
       SUType  _  -> illegalStructUnionErr (posOf cdecl) pos
@@ -1318,13 +1318,15 @@ extractSimpleType isResult pos cdecl  =
 --
 -- * the declaration may have at most one declarator
 --
--- * struct/union types are mapped to `()'
+-- * unknown struct/union types are mapped to `()'
+--
+-- * do *not* take aliases into account
 --
 -- * NB: this is by definition not a result type
 --
-extractPtrType       :: CDecl -> GB ExtType
-extractPtrType cdecl  = do
-  ct <- extractCompType False cdecl
+extractPtrType :: CDecl -> GB ExtType
+extractPtrType cdecl = do
+  ct <- extractCompType False False cdecl
   case ct of
     ExtType et -> return et
     SUType  _  -> return UnitET
@@ -1352,8 +1354,8 @@ extractPtrType cdecl  = do
 --		     `extractCompType' from looking further "into" the
 --		     definition of that pointer.
 --
-extractCompType :: Bool -> CDecl -> GB CompType
-extractCompType isResult cdecl@(CDecl specs declrs ats)  = 
+extractCompType :: Bool -> Bool -> CDecl -> GB CompType
+extractCompType isResult usePtrAliases cdecl@(CDecl specs declrs ats)  = 
   if length declrs > 1 
   then interr "GenBind.extractCompType: Too many declarators!"
   else case declrs of
@@ -1369,13 +1371,15 @@ extractCompType isResult cdecl@(CDecl specs declrs ats)  =
       let declrs' = dropPtrDeclr declr		-- remove indirection
 	  cdecl'  = CDecl specs [(Just declrs', Nothing, Nothing)] ats
           oalias  = checkForOneAliasName cdecl' -- is only an alias remaining?
-      oHsRepr <- case oalias of
+	  osu     = checkForOneCUName cdecl'
+	  oname   = if oalias == Nothing then osu else oalias
+      oHsRepr <- case oname of
 		   Nothing  -> return $ Nothing
 		   Just ide -> queryPtr (True, ide)
       case oHsRepr of
-        Just repr  -> ptrAlias repr             -- got an alias
-	Nothing    -> do			-- no alias => recurs
-	  ct <- extractCompType False cdecl'
+        Just repr | usePtrAliases  -> ptrAlias repr     -- got an alias
+	_                          -> do		-- no alias => recurs
+	  ct <- extractCompType False usePtrAliases cdecl'
 	  returnX $ case ct of
 		      ExtType et -> PtrET et
 		      SUType  _  -> PtrET UnitET
@@ -1401,28 +1405,23 @@ extractCompType isResult cdecl@(CDecl specs declrs ats)  =
 	  traceAlias ide
 	  oHsRepr <- queryPtr (False, ide) -- check for pointer hook alias     
 	  case oHsRepr of
-	    Nothing   -> do		   -- skip current alias (only one)
-			   cdecl' <- getDeclOf ide
-			   let CDecl specs [(declr, init, _)] at =
-			         ide `simplifyDecl` cdecl'
-                               sdecl = CDecl specs [(declr, init, size)] at
-			       -- propagate `size' down (slightly kludgy)
-			   extractCompType isResult sdecl
-	    Just repr -> ptrAlias repr     -- found a pointer hook alias
+	    Just repr | usePtrAliases 
+               -> ptrAlias repr    -- found a pointer hook alias
+	    _  -> do		   -- skip current alias (only one)
+		    cdecl' <- getDeclOf ide
+		    let CDecl specs [(declr, init, _)] at =
+			  ide `simplifyDecl` cdecl'
+			sdecl = CDecl specs [(declr, init, size)] at
+			-- propagate `size' down (slightly kludgy)
+		    extractCompType isResult usePtrAliases sdecl
     --
     -- compute the result for a pointer alias
     --
     ptrAlias (repr1, repr2) = 
       returnX $ DefinedET cdecl (if isResult then repr2 else repr1)
     --
-    -- wrap an `ExtType' into a `CompType' and convert parametrised pointers
-    -- to `Addr' if needed
+    -- wrap an `ExtType' into a `CompType'
     --
-    returnX retval@(PtrET et) = do
-				  keepOld <- getSwitch oldFFI
-				  if keepOld 
-				    then return $ ExtType (PrimET CPtrPT)
-				    else return $ ExtType retval
     returnX retval            = return $ ExtType retval
     --
     tracePtrType = traceGenBind $ "extractCompType: explicit pointer type\n"
@@ -1531,6 +1530,7 @@ specType cpos specs osize =
     bitfieldSpec :: [CTypeSpec] -> ExtType -> Maybe CExpr -> GB CompType
     bitfieldSpec tspecs et (Just sizeExpr) =  -- never called with `Nothing'
       do
+        PlatformSpec {bitfieldIntSignedPS = bitfieldIntSigned} <- getPlatform
         let pos = posOf sizeExpr
 	sizeResult <- evalConstCExpr sizeExpr
 	case sizeResult of
@@ -1597,9 +1597,10 @@ offsetInStruct                :: [CDecl] -> CDecl -> CStructTag -> GB BitSize
 offsetInStruct []    _    _    = return $ BitSize 0 0
 offsetInStruct decls decl tag  = 
   do
+    PlatformSpec {bitfieldAlignmentPS = bitfieldAlignment} <- getPlatform
     (offset, _) <- sizeAlignOfStruct decls tag
     (_, align)  <- sizeAlignOf decl
-    return $ alignOffset offset align
+    return $ alignOffset offset align bitfieldAlignment
 
 -- compute the size and alignment (no padding at the end) of a set of
 -- declarators from a struct
@@ -1608,14 +1609,17 @@ sizeAlignOfStruct :: [CDecl] -> CStructTag -> GB (BitSize, Int)
 sizeAlignOfStruct []    _           = return (BitSize 0 0, 1)
 sizeAlignOfStruct decls CStructTag  = 
   do
+    PlatformSpec {bitfieldAlignmentPS = bitfieldAlignment} <- getPlatform
     (offset, preAlign) <- sizeAlignOfStruct (init decls) CStructTag
     (size, align)      <- sizeAlignOf       (last decls)
-    let sizeOfStruct  = alignOffset offset align `addBitSize` size
+    let sizeOfStruct  = alignOffset offset align bitfieldAlignment
+			`addBitSize` size
 	align'	      = if align > 0 then align else bitfieldAlignment
 	alignOfStruct = preAlign `max` align'
     return (sizeOfStruct, alignOfStruct)
 sizeAlignOfStruct decls CUnionTag   =
   do
+    PlatformSpec {bitfieldAlignmentPS = bitfieldAlignment} <- getPlatform
     (sizes, aligns) <- mapAndUnzipM sizeAlignOf decls
     let aligns' = [if align > 0 then align else bitfieldAlignment
 		  | align <- aligns]
@@ -1628,8 +1632,9 @@ sizeAlignOfStruct decls CUnionTag   =
 sizeAlignOfStructPad :: [CDecl] -> CStructTag -> GB (BitSize, Int)
 sizeAlignOfStructPad decls tag =
   do
+    PlatformSpec {bitfieldAlignmentPS = bitfieldAlignment} <- getPlatform
     (size, align) <- sizeAlignOfStruct decls tag
-    return (alignOffset size align, align)
+    return (alignOffset size align bitfieldAlignment, align)
 
 -- compute the size and alignment constraint of a given C declaration
 --
@@ -1640,18 +1645,30 @@ sizeAlignOf       :: CDecl -> GB (BitSize, Int)
 --
 sizeAlignOf cdecl  = 
   do
-    ct <- extractCompType False cdecl
+    ct <- extractCompType False False cdecl
     case ct of
-      ExtType (FunET _ _        ) -> return (bitSize CFunPtrPT, 
-					     alignment CFunPtrPT)
+      ExtType (FunET _ _        ) -> do
+				       align <-	alignment CFunPtrPT
+				       return (bitSize CFunPtrPT, align)
       ExtType (IOET  _          ) -> interr "GenBind.sizeof: Illegal IO type!"
       ExtType (PtrET t          ) 
-        | isFunExtType t          -> return (bitSize CFunPtrPT, 
-					     alignment CFunPtrPT)
-        | otherwise		  -> return (bitSize CPtrPT, alignment CPtrPT)
-      ExtType (DefinedET _ _    ) -> return (bitSize CPtrPT, alignment CPtrPT)
+        | isFunExtType t          -> do
+				       align <-	alignment CFunPtrPT
+				       return (bitSize CFunPtrPT, align)
+        | otherwise		  -> do
+				       align <- alignment CPtrPT
+				       return (bitSize CPtrPT, align)
+      ExtType (DefinedET _ _    ) -> 
+        interr "GenBind.sizeAlignOf: Should never get a defined type"
+{- OLD:
+                                     do
+				       align <- alignment CPtrPT
+				       return (bitSize CPtrPT, align)
         -- FIXME: The defined type could be a function pointer!!!
-      ExtType (PrimET pt        ) -> return (bitSize pt, alignment pt)
+ -}
+      ExtType (PrimET pt        ) -> do
+				       align <- alignment pt
+				       return (bitSize pt, align)
       ExtType UnitET              -> voidFieldErr (posOf cdecl)
       SUType su                   -> 
         do
@@ -1678,15 +1695,18 @@ sizeAlignOf cdecl  =
 -- * if the alignment constraint is negative or zero, it is the alignment
 --   constraint for a bitfield
 --
-alignOffset :: BitSize -> Int -> BitSize
-alignOffset offset@(BitSize octetOffset bitOffset) align 
+-- * the third argument gives the platform-specific bitfield alignment
+--
+alignOffset :: BitSize -> Int -> Int -> BitSize
+alignOffset offset@(BitSize octetOffset bitOffset) align bitfieldAlignment
   | align > 0 && bitOffset /= 0 =		-- close bitfield first
-    alignOffset (BitSize (octetOffset + (bitOffset + 7) `div` 8) 0) align
+    alignOffset (BitSize (octetOffset + (bitOffset + 7) `div` 8) 0) align 
+		bitfieldAlignment
   | align > 0 && bitOffset == 0 =	        -- no bitfields involved
     BitSize (((octetOffset - 1) `div` align + 1) * align) 0
   | bitOffset == 0 	        	        -- start a bitfield
     || overflowingBitfield	=		-- .. or overflowing bitfield
-    alignOffset offset bitfieldAlignment
+    alignOffset offset bitfieldAlignment bitfieldAlignment
   | otherwise			=		-- stays in current bitfield
     offset
   where
