@@ -3,5 +3,5 @@
 struct C2HS_COND_SENTRY_1;
 #else
 #endif
-#c
-#endc
+int fooC (int, int);
+
